//
//    FILE: dht11.cpp
// VERSION: 0.3.2
// PURPOSE: DHT11 Temperature & Humidity Sensor library for Arduino
// LICENSE: GPL v3 (http://www.gnu.org/licenses/gpl.html)
//
// DATASHEET: http://www.micro4you.com/files/sensor/DHT11.pdf
//
// HISTORY:
// George Hadjikyriacou - Original version (??)
// Mod by SimKard - Version 0.2 (24/11/2010)
// Mod by Rob Tillaart - Version 0.3 (28/03/2011)
// + added comments
// + removed all non DHT11 specific code
// + added references
//

#include "dht11.h"

// <<constructor>>  
dht11::dht11(){

  // awaiting measure command
  pinMode(DHT11_PIN, OUTPUT); 
  digitalWrite(DHT11_PIN, HIGH);
}

// <<destructor>>
dht11::~dht11(){
}

// return values: 0=OK, -1=checksum error, -2=timeout
int dht11::measure()
{
  unsigned int loopCnt;
  unsigned long start;    // Bit signal start time, in micros

  uint8_t information[5]; // Buffer to receive dht11 data
  uint8_t byte_index = 0; // Index in data array (0 to 4)
  uint8_t bit_count = 7;  // Bit index in information[byte_index] (7 to 0)
  uint8_t checksum;       // Information checksum control

  // Reset properties
  humidity    = 0;        // dry atmosphere...
  temperature = -273;     // pretty cold too... 

  // Empty buffer
  for (int i=0; i< 5; i++) 
    information[i] = 0;

  // MCU Sends out Start Signal (step 1 : LOW 18ms)
  pinMode(DHT11_PIN, OUTPUT);
  digitalWrite(DHT11_PIN, LOW);
  delay(18);

  // MCU Sends out Start Signal (step 2 : HIGH 40 micros)
  digitalWrite(DHT11_PIN, HIGH);
  delayMicroseconds(40);

  // DHT Response (step 1 : LOW 80 micros)
  pinMode(DHT11_PIN, INPUT);
  loopCnt = 10000;
  while(digitalRead(DHT11_PIN) == LOW)
    if (loopCnt-- == 0) 
      return MEASURE_FAILED;

  // DHT Response (step 2 : HIGH 80 micros)
  loopCnt = 10000;
  while(digitalRead(DHT11_PIN) == HIGH)
    if (loopCnt-- == 0) 
      return MEASURE_FAILED;

  // READ OUTPUT - 40 BITS => 5 BYTES or TIMEOUT
  for (int i=0; i<40; i++) {

    // DHT Response (Start transmit = LOW 50 micros)
    loopCnt = 10000;
    while(digitalRead(DHT11_PIN) == LOW)
      if (loopCnt-- == 0) 
        return MEASURE_FAILED;

    // DHT Response (Bit transmit = HIGH 26 to 70 micros)
    start = micros();
    loopCnt = 10000;
    while(digitalRead(DHT11_PIN) == HIGH)
      if (loopCnt-- == 0) 
        return MEASURE_FAILED;

    // 26-28 micros HIGH signal means "0", 70 micros HIGH signal means "1"
    if ((micros() - start) > 50) 
      information[byte_index] |= (1 << bit_count);

    // Next bit
    if (bit_count == 0) {
      // Next byte
      byte_index++; 
      bit_count = 7;    
    }
    else 
      bit_count--;
  }

  // Save results
  humidity    = information[0]; // DHT11 accuracy is 1%, so information[1] is allways 0.
  temperature = information[2]; // DHT11 accuracy is 1°C, so information[3] is allways 0.

   // Checksum control
  checksum = information[0] + information[2];  
  if ( information[4] != checksum ) 
    return MEASURE_BAD;

  // Sample done
  return MEASURE_OK;
  
}

//
// END OF FILE
//






