
#include "leds.h"
#include "dht11.h"
#include "hp82165a.h"

/************************************************
 * Arduino HP-IL GPIO Interface example
 *************************************************/
dht11     Sensor;
leds      Leds; 
hp82165a  Gpio;

const int RELAY1_PIN  = 39;    

/************************************************
 * Setup
 ************************************************/
void setup() {

  // Serial.println("Setup...");   
  Serial.begin(9600);
  
  pinMode(RELAY1_PIN, OUTPUT);
  digitalWrite(RELAY1_PIN, LOW);

}

/************************************************
 * Loop
 ************************************************/
void loop() {

  // Process GPIO signals
  switch ( Gpio.watch() ) {

  case WATCH:

    // Continuous watch
    break;

  case COMMAND:

    // HP-IL listener data received
    execute( Gpio.enter() );
    break;

  case TRIGGER:

    // HP-IL Group Execute Trigger received
    trigger();   
    break;

  case SERVICE:

    // HP-IL interface service request event 
    // Caution : This event occurs periodically when interface is off but plugged
    failure();
    break;

  } // end switch watch

}

/************************************************
 * GPIO Command execution
 ************************************************/
void execute(String data) {

  // Command protocol : 
  // Acknowledge (OK) when command is processed, 
  // No acknowledge (KO) if any problem occured

  /*
  Serial.print("Command : [");
  Serial.print(data);
  Serial.println("]");
  */
  
  // Check for known commands
  if ( data.equals("SRQ") ) { 
    
    // Test command to check SRQ interrupts
    Gpio.service();
    failure();
  }

  else if ( data.equals("MARCHE") || data.equals("ON") ) { 
    Leds.on();
    ack();
  }

  else if ( data.equals("NOIR") || data.equals("BLACK") || data.equals("ARRET") || data.equals("OFF") ) { 
    Leds.off();
    ack();
  }

  else if ( data.equals("VERT") || data.equals("GREEN") ) { 
    Leds.green();
    ack();
  }

  else if ( data.equals("ORANGE") || data.equals("JAUNE") || data.equals("YELLOW") ) { 
    Leds.yellow();
    ack();
  }

  else if ( data.equals("ROUGE") || data.equals("RED") ) { 
    Leds.red();
    ack();
  }

  else if ( data.equals("CLIGNOTE") || data.equals("BLINK") ) { 
    Leds.blink(200);
    ack();
  }

  else if ( data.equals("UP1") || data.equals("ALLUME") || data.equals("JOUR") ) { 
    Leds.yellow();
    digitalWrite(RELAY1_PIN, HIGH);
    ack();
  }

  else if ( data.equals("DOWN1") || data.equals("ETEINT") || data.equals("NUIT") ) { 
    Leds.off();
    digitalWrite(RELAY1_PIN, LOW);
    ack();
  }


  else {

    // No Acknowledge
    nak();

    // Serial.println("ERROR : Unknown command...");

  } // end if known command

}

/************************************************
 * GPIO Trigger execution
 ************************************************/
void trigger() {

  //String trace;
  char response[32];
 
  // Serial.println("Measuring...");

  switch ( Sensor.measure() ){
  case MEASURE_OK :

    // Trigger protocol reply (N+2 rows) 
    // Row 1 = Acknowledge, Row 2 = Number of values (N), Row 3 = Value1, ..., Row N+2 = Value N 
    sprintf(response,"OK\r\n2\r\n%d\r\n%d\r\n",Sensor.humidity,Sensor.temperature);
    Gpio.output(response);
    
    //      trace = "Humidity = ";
    //      trace += Sensor.humidity;
    //      trace += "%, Temperature = ";
    //      trace += Sensor.temperature;
    //      trace += " C";
    //      Serial.println(trace);

    break;

  default :

    // No Acknowledge : Checksum error (data may not be correct) or Timeout error (no valid data) 
    nak();

    // Request service 
    // NB : HP-IL Controller periodically sends IDY frame to catch HP82165A SRQ status bit interrupts. 
    Gpio.service();

    // Serial.println("ERROR : Sensor malfunction...");

    break;

  } // end switch measure

}

/************************************************
 * GPIO Acknowledgement
 ************************************************/
void ack() {
  Gpio.output("OK");
}

/************************************************
 * GPIO Negative acknowledgement
 ************************************************/
void nak() {
  Gpio.output("KO");
}

/************************************************
 * GPIO failure
 ************************************************/
void failure() {
  digitalWrite(RELAY1_PIN, LOW);
  Leds.blink(50);
}

